/******************************************************************/
/*                                                                */
/* Module:       jb_io.c                                          */
/*                                                                */
/* Descriptions: Manages I/O related routines, file and string    */
/*               processing functions.                            */
/*                                                                */
/* Revisions:    .0 02/22/02                                      */
/*               .1 04/11/02                                      */
/*               A new function, jb_grabdata() has been added     */
/*               .2 07/16/03                                      */
/*               VerifyHardware() function has been splitted into */
/*               two functions, VerifyBBII() and VerifyBBMV()     */
/*               .3 07/01/2004                                    */
/*                                                                */
/******************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setupapi.h>         /* ޲ײތpɒǉ */
#include "jb_io.h"
#include "jb_jtag.h"

/* ---> ޲ײތpɒǉ */
#pragma once
#pragma comment(lib,"setupapi.lib")
#pragma comment(lib,"comctl32.lib")
/* <--- ޲ײތpɒǉ */

#define MAXBUFF         (1024)

// target PORT t
// |---|---|PA5|PA4|PA3|PA2|PA1|PA0|
// |---|---|---|---|---|TDI|TMS|TCK|
// |PB7|PB6|PB5|PB4|PB3|PB2|PB1|PB0|
// |---|---|---|---|---|LED|---|TDO|
// |PC7|PC6|PC5|PC4|PC3|PC2|PC1|PC0|
// |---|---|---|---|---|---|---|---|


extern const int sig_port_maskbit[4][2];


/*--------------------------------------------------------------------
  USB
  --------------------------------------------------------------------*/

HANDLE  hCom = INVALID_HANDLE_VALUE;


typedef union outPortUnion {
    unsigned char value;
    struct opBitsStr {
        unsigned char tck:1;      // tdo
        unsigned char tms:1;
        unsigned char tdi:1;
        unsigned char bit3:1;
        unsigned char bit4:1;
        unsigned char bit5:1;
        unsigned char bit6:1;
        unsigned char bit7:1;
    } bits;
} outPortType;

outPortType out_word;


int     gnTxd;
//LPBYTE  gbTxd;
BYTE  gbTxd[MAXBUFF];

DWORD   gdwTck;
DWORD   gdwTdo;



/*
* GUID ׽̂B
* ݁FTRUE=AFALSE=ُ
*/
static BOOL WINAPI CheckClassValues(
GUID *lpcg) {       // GUID ߲
    CHAR   szGUID[512];
    CHAR   szClassName[512];
    LPGUID pGUID = lpcg;
    sprintf(szGUID,
             "{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}",
             pGUID->Data1,
             pGUID->Data2,
             pGUID->Data3,
             pGUID->Data4[0],
             pGUID->Data4[1],
             pGUID->Data4[2],
             pGUID->Data4[3],
             pGUID->Data4[4],
             pGUID->Data4[5],
             pGUID->Data4[6],
             pGUID->Data4[7]);
    if((SetupDiClassNameFromGuid(lpcg,
                                 szClassName,
                                 sizeof(szClassName),
                                 NULL))&&
       (!strcmp(szClassName,"Ports"))) {
        return(TRUE);
    }
    return(FALSE);
}
/*
* ذ̂COM߰Ĕԍ擾B
* ݁F-1!= ̂ƂA-1==̂Ƃُ
*/
static int WINAPI GetComPortNum(VOID) {
    /*--- pذ ---*/
    static CHAR gszDeviceDesc[] = "Communications Port";
    /*--- ߰Ă̸׽ID ---*/
    static GUID CYUSBDRV_GUID = {
        0x4D36E978, 0xE325, 0x11CE, 0xBF, 0xC1, 0x08, 0x00, 0x2B, 0xE1, 0x03, 0x18
    };
    static SP_CLASSIMAGELIST_DATA ClassImageListData;
    CHAR             szClassDesc[512];
    CHAR             szDeviceDesc[512];
    GUID             ClassGuid;
    HDEVINFO         hDevInfo;
    SP_DEVINFO_DATA  DevInfoData;
    int              i;
    BOOL             bSubTree;
    UINT             iImage;
    CHAR            *lps;
    int              rtc = -1;

    hDevInfo = SetupDiGetClassDevs(&CYUSBDRV_GUID,
                                   NULL,
                                   NULL,
                                   DIGCF_PRESENT);
    if(hDevInfo == INVALID_HANDLE_VALUE) {
        return(-1);
    }
    SetupDiGetClassDescription(&CYUSBDRV_GUID,szClassDesc,sizeof(szClassDesc),NULL);
    SetupDiGetClassImageIndex(&ClassImageListData,&CYUSBDRV_GUID,&iImage);
    bSubTree = TRUE;
    DevInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
    for (i = 0;SetupDiEnumDeviceInfo(hDevInfo,i,&DevInfoData);i++) {
        ClassGuid = DevInfoData.ClassGuid;
        if(bSubTree) {
            if(CheckClassValues(&ClassGuid) != TRUE) {
                break;
            }
            bSubTree = FALSE;
        }
        if(!SetupDiGetDeviceRegistryProperty(
                hDevInfo,
                &DevInfoData,
                SPDRP_FRIENDLYNAME,
                NULL,
                szDeviceDesc,
                sizeof(szDeviceDesc),
                NULL)) {
            if(!SetupDiGetDeviceRegistryProperty(
                    hDevInfo,
                    &DevInfoData,
                    SPDRP_DEVICEDESC,
                    NULL,
                    szDeviceDesc,
                    sizeof(szDeviceDesc),
                    NULL)) {
                strcpy(szDeviceDesc,"No Device Names");
            }
        }

        if((!strncmp(szDeviceDesc,gszDeviceDesc,strlen(gszDeviceDesc)))&&
           ((lps = strstr(szDeviceDesc,"(COM")) != 0)) {
            rtc = strtoul(&lps[4],0,0);
            break;
        }
    }
    SetupDiDestroyDeviceInfoList(hDevInfo);
    return(rtc);
}
/*
* MްCOM߰Ăo͂̂҂B
*/
static void WINAPI WaitCommTxEmpty(void) {
    DWORD       dwHldMask;
    DWORD       dwEvtMask;
    OVERLAPPED  os;

    GetCommMask(hCom,&dwHldMask);
    memset(&os,0,sizeof(OVERLAPPED));
    // create I/O event used for overlapped read
    os.hEvent = CreateEvent(NULL,    // no security
                            TRUE,    // explicit reset req
                            FALSE,   // initial event reset
                            NULL); // no name
    if(os.hEvent == NULL) {
        MessageBox(NULL,"Failed to create event for thread!","COM Error!",
                   MB_ICONEXCLAMATION | MB_OK);
        Sleep(2);
    } else {
        if(!SetCommMask(hCom,EV_TXEMPTY)) {
            Sleep(2);
        } else {
            do {
                dwEvtMask = 0;
                WaitCommEvent(hCom,&dwEvtMask,NULL);
            } while(!(dwEvtMask & EV_TXEMPTY));
            SetCommMask(hCom,dwHldMask);
        }
        // get rid of event handle
        CloseHandle(os.hEvent);
    }
}
/*
* Mް64޲Ăɋ؂AޯĂ֑B
*/
static VOID WINAPI usb_send(LPBYTE lpb,DWORD cbb) {
    DWORD  cbWrite;
    DWORD  loc;
    DWORD  blk;

    for(loc = 0;loc < cbb;) {
        blk = ((cbb - loc) > 256)?  256 : (cbb - loc);
        cbWrite = 0;
        WriteFile(hCom,(LPVOID)&lpb[loc],blk,&cbWrite,NULL);
        loc += cbWrite;
//        WaitCommTxEmpty();
//        Sleep(2);
    }
}
/*
* obt@ɕۑĂf[^𑗐MB
*/
static VOID WINAPI FlushTxBuf(VOID) {
    if(gnTxd > 0) {
        usb_send(gbTxd,gnTxd);
        gnTxd = 0;
    }
}
/*
* obt@ɑMf[^ۑB
* TCY𒴂ꍇACɑoB
*/
VOID WINAPI UpdateTxBuf(BYTE txd) {

gdwTck++;

    gbTxd[gnTxd++] = txd;
    if(gnTxd >= MAXBUFF) {
        usb_send(gbTxd,gnTxd);
        gnTxd = 0;
    }
}
/*
* COM|[gI[vB
*/
static BOOL WINAPI OpenComm(VOID) {
    int   rtc;
    CHAR  szBuf[32];

    rtc = GetComPortNum();
    if(rtc < 0) {
        printf("COM port open error\n");

        exit(1);

        return(FALSE);
    }
    wsprintf(szBuf,"\\\\.\\COM%d",rtc);
    hCom = CreateFile(szBuf,
                      GENERIC_READ | GENERIC_WRITE,
                      0,
                      NULL,
                      OPEN_EXISTING,
                      0,
                      NULL);
    if(hCom == INVALID_HANDLE_VALUE) {
        printf("can not open #<%s>\r\n",szBuf);
        exit(1);
    }
    gnTxd = 0;
    return(TRUE);
}
/*
* COM|[gN[YB
*/
static BOOL WINAPI CloseComm(VOID) {
    if(hCom != INVALID_HANDLE_VALUE) {
        CloseHandle(hCom);
        hCom = INVALID_HANDLE_VALUE;
    }
    return(TRUE);
}
/*
* TDOrbg擾B
* ߂ĺA_̌ʂƂB
*/
/* read the TDO bit from port */
unsigned char WINAPI readTDOBit(void) {
    BYTE   brxd;
    DWORD  cbRead;
    int    retry;

    UpdateTxBuf(0x80);       // TDO=1
    FlushTxBuf();

//    Sleep(1);

gdwTdo++;

    for(cbRead = 0,retry = 100;(!cbRead)&&(retry > 0);retry--,Sleep(2)) {
        ReadFile(hCom,(LPVOID)&brxd,1,&cbRead,NULL);
    }
    return((brxd & 0x01)?  1 : 0);
}



#if PORT==WINDOWS_NT
#include <windows.h>
#endif /* PORT==WINDOWS_NT */


#define MAX_FILE_LINE_LENGTH            160



#if PORT==WINDOWS_NT
/******************************************************************/
/* Name:         InitNtDriver                                     */
/*                                                                */
/* Parameters:   None.                                            */
/*                                                                */
/* Return Value: None.                                            */
/*                                                                 */
/* Descriptions: Initialize Windows NT Driver for ByteBlasterMV.  */
/*                                                                */
/******************************************************************/
void InitNtDriver() {
    OpenComm();
}

/******************************************************************/
/* Name:         CloseNtDriver                                    */
/*                                                                */
/* Parameters:   None.                                            */
/*                                                                */
/* Return Value: None.                                            */
/*                                                                 */
/* Descriptions: Close Windows NT Driver.                         */
/*                                                                */
/******************************************************************/
void CloseNtDriver() {
    CloseComm();
}

/******************************************************************/
/* Name:         flush_ports                                      */
/*                                                                */
/* Parameters:   None.                                            */
/*                                                                */
/* Return Value: None.                                            */
/*                                                                 */
/* Descriptions: Flush processes in port_io_buffer and reset      */
/*               buffer size to 0.                                */
/*                                                                */
/******************************************************************/
void flush_ports(void) {
    FlushTxBuf();
}

/******************************************************************/
/* Name:         VerifyBBII (ByteBlaster II)                      */
/*                                                                */
/* Parameters:   None.                                            */
/*                                                                */
/* Return Value: '0' if verification is successful;'1' if not.    */
/*                                                                 */
/* Descriptions: Verify if ByteBlaster II is properly attached to */
/*               the parallel port.                               */
/*                                                                */
/******************************************************************/
int VerifyBBII() {
    int error = 0;
    int test_count = 0;
    int read_data = 0;

    return(0);

    for ( test_count = 0; test_count < 2; test_count++ ) {
        /* Write '0' to Pin 6 (Data4) for the first test and '1' for the second test */
        int vector = (test_count) ? 0x10 : 0x0;/* 0001 0000:0000 0000... drive to Port0 */
        int expect = (test_count) ? 0x40 : 0x0;/* X1XX 0XXX:X0XX 0XXX... expect from Port1 */

        WritePort( PORT_0, vector, BUFFER_OFF );

        /* Expect '0' at Pin 10 (Ack) and Pin 15 (Error) for the first test */
        /* and '1' at Pin 10 (Ack) and '0' Pin 15 (Error) for the second test */
        read_data = ReadPort( PORT_1 ) & 0x40;

        /* If no ByteBlaster II detected, error = 1 */
        if (test_count==0) {
            if(read_data==0x00)
                error=0;
            else error=1;
        }

        if (test_count==1) {
            if(read_data==0x40)
                error=error|0;
            else error=1;
        }
    }

    
    if (!error) {
        fprintf( stdout, "Info: Verifying hardware: ByteBlaster II found...\n" );
        return error;
    }
    else
        return error;
}

/******************************************************************/
/* Name:         VerifyBBMV (ByteBlasterMV)                       */
/*                                                                */
/* Parameters:   None.                                            */
/*                                                                */
/* Return Value: '0' if verification is successful;'1' if not.    */
/*                                                                 */
/* Descriptions: Verify if ByteBlasterMV is properly attached to  */
/*               the parallel port.                               */
/*                                                                */
/******************************************************************/
int VerifyBBMV() {
    int error = 0;
    int test_count = 0;
    int read_data = 0;

    return(0);

    for ( test_count = 0; test_count < 2; test_count++ ) {
        /* Write '0' to Pin 7 and Pin 9 (Data5,7) for the first test and '1' for the second test */
        int vector = (test_count) ? 0xA0 : 0x0;/* 1010 0000:0000 0000... drive to Port0 */
        int expect = (test_count) ? 0x60 : 0x0;/* X11X XXXX:X00X XXXX... expect from Port1 */

        WritePort( PORT_0, vector, BUFFER_OFF );

        /* Expect '0' at Pin 10 and Pin 12 (Ack and Paper End) for the first test and '1' for the second test */
        read_data = ReadPort( PORT_1 ) & 0x60;

        /* If no ByteBlasterMV detected, error = 1 */
        if (test_count==0) {
            if(read_data==0x00)
                error=0;
            else error=1;
        }

        if (test_count==1) {
            if(read_data==0x60)
                error=error|0;
            else error=1;
        }
    }

    if (!error) {
        fprintf( stdout, "Info: Verifying hardware: ByteBlasterMV found...\n" );
        return error;
    }
    else
        return error;
}

#endif /* PORT==WINDOWS_NT */

/******************************************************************/
/* Name:         ReadPort                                         */
/*                                                                */
/* Parameters:   port                                             */
/*               -the index of port from the parallel port base   */
/*                address.                                        */
/*                                                                */
/* Return Value: Value of the port.                               */
/*                                                                 */
/* Descriptions: Read the value of the port registers.            */
/*                                                                */
/******************************************************************/
int ReadPort(int port) {
    // invert
    return(readTDOBit()?  0: sig_port_maskbit[SIG_TDO][1]);
}


/******************************************************************/
/* Name:         WritePort                                        */
/*                                                                */
/* Parameters:   port,data,buffer_enable                          */
/*               -port is the index from the parallel port base   */
/*                address.                                        */
/*               -data is the value to dump to the port.          */
/*               -purpose of write.                               */
/*                                                                */
/* Return Value: None.                                            */
/*                                                                 */
/* Descriptions: Write "data" to "port" registers. When dump to   */
/*               port 0,if "buffer_enable"=1, processes in          */
/*                 "port_io_buffer" are flushed when                  */
/*               "PORT_IO_BUFFER_SIZE" is reached                  */
/*               If "buffer_enable"=0,"data" is dumped to port 0  */
/*               at once.                                         */
/*                                                                */
/******************************************************************/
void WritePort(int port,int data,int buffer_enable) {
    out_word.value = 0;
    if(data & sig_port_maskbit[SIG_TCK][1]) {
        out_word.bits.tck = 1;
    }
    if(data & sig_port_maskbit[SIG_TMS][1]) {
        out_word.bits.tms = 1;
    }
    if(data & sig_port_maskbit[SIG_TDI][1]) {
        out_word.bits.tdi = 1;
    }
    UpdateTxBuf((BYTE)out_word.value);
    if(port != 0 || buffer_enable != BUFFER_ON) {
        FlushTxBuf();
    }
}

/*****************************/
/*                           */
/* File processing functions */
/*                           */
/*****************************/

int jb_fopen(char* argv,char* mode) {
    FILE* file_id;

    file_id = fopen( argv, mode );

    return (int) file_id;
}

int jb_fclose(int file_id) {
    fclose( (FILE*) file_id);

    return 0;
}

int jb_fseek(int finputid,int start,int end) {
    int seek_position;

    seek_position = fseek( (FILE*) finputid, start, end );

    return seek_position;
}

int jb_ftell(int finputid) {
    int file_size;

    file_size = ftell( (FILE*) finputid );

    return file_size;
}

int jb_fgetc(int finputid) {
    int one_byte;

    one_byte = fgetc( (FILE*) finputid );

    return one_byte;
}

char* jb_fgets(char* buffer, int finputid) {
    char* test;
    test=fgets(buffer,MAX_FILE_LINE_LENGTH,(FILE*) finputid);

    return test;
}

/*******************************/
/*                             */
/* String processing functions */
/*                             */
/*******************************/

/******************************************************************/
/* Name:         jb_grabdata                                      */
/*                                                                */
/* Parameters:   buffer, start_byte, term, str                    */
/*               -buffer is the line buffer stored when reading   */
/*                the CDF.                                        */
/*               -start_byte is the byte to start with on the     */
/*                buffer parsed from CDF                          */
/*               -term is how many terms to look into from the    */
/*                start_byte parsed from CDF                      */
/*                 -str is the string from start_byte till term in  */
/*                  the buffer parsed from CDF                      */
/*                                                                */
/* Return Value: mark - "0" if all spaces in the buffer, else "1" */
/*                                                                 */
/******************************************************************/
/* [sbng,4/12/02,jb_io.c ver1.1] New function added */
int jb_grabdata(char* buffer,int start_byte,int term,char* str) {
    unsigned i=0,j=0;
    int mark=0;
    int space=0;

    if(start_byte<0 || start_byte>=(int)(strlen(buffer)-1)) {
        str[0]='\0';
        return (-1);
    }

    for(i=start_byte;i<strlen(buffer);i++) {
        if(mark==0) {
            if( buffer[i]!=' ' && buffer[i]!='\t' && buffer[i]!='\n' ) {
                if(space==term-1)
                    str[j++]=buffer[i];
                mark=1;
            } else if( buffer[i]==' ' || buffer[i]=='\t' ) {
                mark=0;
            } else {
                if(!((buffer[i]==' ' || buffer[i]=='\t') && (buffer[(i>0)? i-1:0]==' ' || buffer[i]=='\t' )))
                    space++;
                if(space>term-1)
                    break;
            }
        } else if(mark==1) {
            if( buffer[i]!=' ' && buffer[i]!='\t' && buffer[i]!='\n' ) {
                if(space==term-1)
                    str[j++]=buffer[i];
            } else {
                if(!((buffer[i]==' ' || buffer[i]=='\t' ) && (buffer[(i>0)? i-1:0]==' ' || buffer[i]=='\t' )))
                    space++;
                if(space>term-1)
                    break;
            }
        }
        else;
    }

    str[j]='\0';

    return mark;
}

void jb_strcpy(char *a,char *b) {
    strcpy(a,b);
}

int jb_str_cmp(char* charset,char* buffer) {
    char* pc;
    char  temp[MAX_FILE_LINE_LENGTH+1],store;
    unsigned int i;

    for(i=0;i<strlen(buffer);i++) {
        strcpy(temp,buffer);
        pc = &temp[i];

        if(*pc==*charset) {
            store = temp[i+strlen(charset)];
            temp[i+strlen(charset)]='\0';

            if(!strncmp(pc,charset,strlen(charset))) {
                temp[i+strlen(charset)]=store;
                return i+1;
            }

            temp[i+strlen(charset)]=store;
        }

        pc++;
    }

    return 0;
}

int jb_strlen(const char* str) {
    return strlen(str);
}

/******************************************************************/
/* Name:         jb_strcmp                                        */
/*                                                                */
/* Parameters:   a, b                                              */
/*               -a and b are strings                              */
/*                                                                */
/* Return Value: -0 if a and b are identical                      */
/*                 -!0 if a and b are not identical                  */
/*                                                                 */
/* Descriptions: Used to compare strings                          */
/*                                                                */
/******************************************************************/
int jb_strcmp(char *a,char *b) {
    return strcmp(a,b);
}

void jb_strcat(char *dst,char *src) {
    strcat(dst,src);
}

int jb_atoi(char* number) {
    return(atoi(number));
}

void jb_toupper(char* str) {
    char* pstr;

    pstr = str;
    while(*pstr) {
        if(*pstr >= 'a' && *pstr <= 'z') {
            *pstr -= 0x20;
        }
        pstr++;
    }
}